if (type(AzOptionsFactory) == "table") and (AzOptionsFactory.vers >= 4) then
	return;
end

AzOptionsFactory = {
	vers = 4,
	objectNames = {},
	objectOffsetX = { Check = 10, Slider = 18, Color = 14, DropDown = 136, Text = 142 },
	objectOffsetY = { Check = -4, Slider = 4, Color = 6, DropDown = 2, Text = 0 },
};

local _G = getfenv(0);
local ReturnZero = function() return 0; end

--------------------------------------------------------------------------------------------------------
--                                          Helper Functions                                          --
--------------------------------------------------------------------------------------------------------

-- Converts string colors to RGBA
function AzOptionsFactory:HexStringToRGBA(string)
	local a, r, g, b = string:match("^|c(..)(..)(..)(..)");
	return format("%d","0x"..r) / 255, format("%d","0x"..g) / 255, format("%d","0x"..b) / 255, format("%d","0x"..a) / 255;
end

-- Generate Unique Object Name
local function GenerateObjectName(type)
	AzOptionsFactory.objectNames[type] = (AzOptionsFactory.objectNames[type] or 0) + 1;
	return "AzOptionsFactory"..type..AzOptionsFactory.objectNames[type];
end

-- Gets an existing object or creates a new one if needed
function AzOptionsFactory:GetObject(type)
	local index = (self.objectUse[type] + 1);
	if (not self[type]) then
		AzMsg("ERROR! invalid object name");
		return;
	elseif (self.objects[type]) and (self.objects[type][index]) then
		self.objectUse[type] = index;
		return self.objects[type][index];
	else
		self.objectUse[type] = index;
		local obj = self[type](self);
		if (not self.objects[type]) then
			self.objects[type] = {};
		end
		tinsert(self.objects[type],obj);
		return obj;
	end
end

-- Resets the object use and hides all objects
function AzOptionsFactory:ResetObjectUse()
	wipe(self.objectUse);
	for _, table in pairs(self.objects) do
		for _, obj in ipairs(table) do
			obj:Hide();
		end
	end
end

-- Creates New Factory Instance
function AzOptionsFactory:New(owner,cfg,ChangeSettingFunc)
	local fac = {
		ChangeSettingFunc = ChangeSettingFunc,

		owner = owner,
		cfg = cfg,

		objects = {},
		objectUse = {},
	};
	setmetatable(fac,{ __index = AzOptionsFactory });
	setmetatable(fac.objectUse,{ __index = ReturnZero });
	return fac;
end

--------------------------------------------------------------------------------------------------------
--                                            Slider Frame                                            --
--------------------------------------------------------------------------------------------------------

-- EditBox Enter Pressed
local function SliderEdit_OnEnterPressed(self)
	self:GetParent().slider:SetValue(self:GetNumber());
end

-- Slider Value Changed
local function Slider_OnValueChanged(self)
	local parent = self:GetParent();
	parent.edit:SetNumber(self:GetValue());
	parent.factory:ChangeSettingFunc(parent.option.var,self:GetValue());
end

-- OnMouseWheel
local function Slider_OnMouseWheel(self)
	self:SetValue(self:GetValue() + self:GetParent().option.step * arg1);
end

-- New Slider
function AzOptionsFactory:Slider()
	local f = CreateFrame("Frame",nil,self.owner);
	f:SetWidth(292);
	f:SetHeight(32);

	f.edit = CreateFrame("EditBox",GenerateObjectName("EditBox"),f,"InputBoxTemplate");
	f.edit:SetWidth(45);
	f.edit:SetHeight(21);
	f.edit:SetPoint("BOTTOMLEFT");
	f.edit:SetScript("OnEnterPressed",SliderEdit_OnEnterPressed);
	f.edit:SetAutoFocus(nil);
	f.edit:SetMaxLetters(4);

	local sliderName = GenerateObjectName("Slider");
	f.slider = CreateFrame("Slider",sliderName,f,"OptionsSliderTemplate");
	f.slider:SetPoint("TOPLEFT",f.edit,"TOPRIGHT",5,-3);
	f.slider:SetPoint("BOTTOMRIGHT",0,-2);
	f.slider:SetScript("OnValueChanged",Slider_OnValueChanged);
	f.slider:SetScript("OnMouseWheel",Slider_OnMouseWheel);
	f.slider:EnableMouseWheel(1);

	f.text = _G[sliderName.."Text"];
	f.text:SetTextColor(1.0,0.82,0);
	f.low = _G[sliderName.."Low"];
	f.low:ClearAllPoints();
	f.low:SetPoint("BOTTOMLEFT",f.slider,"TOPLEFT",0,0);
	f.high = _G[sliderName.."High"];
	f.high:ClearAllPoints();
	f.high:SetPoint("BOTTOMRIGHT",f.slider,"TOPRIGHT",0,0);

	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                            Check Button                                            --
--------------------------------------------------------------------------------------------------------

local function CheckButton_OnEnter(self)
	self.text:SetTextColor(1,1,1);
	if (self.option.tip) then
		GameTooltip:SetOwner(self,"ANCHOR_RIGHT");
		GameTooltip:AddLine(self.option.label,1,1,1);
		GameTooltip:AddLine(self.option.tip,nil,nil,nil,1);
		GameTooltip:Show();
	end
end

local function CheckButton_OnLeave(self)
	self.text:SetTextColor(1,0.82,0);
	GameTooltip:Hide();
end

local function CheckButton_OnClick(self)
	self.factory:ChangeSettingFunc(self.option.var,self:GetChecked() == 1);
end

-- New CheckButton
function AzOptionsFactory:Check()
	local f = CreateFrame("CheckButton",nil,self.owner,"OptionsSmallCheckButtonTemplate");
	f:SetWidth(26);
	f:SetHeight(26);
	f:SetScript("OnEnter",CheckButton_OnEnter);
	f:SetScript("OnClick",CheckButton_OnClick);
	f:SetScript("OnLeave",CheckButton_OnLeave);

	f.text = select(6,f:GetRegions());

	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                            Color Button                                            --
--------------------------------------------------------------------------------------------------------

local CPF = ColorPickerFrame;

-- Color Picker Function
local function ColorButton_ColorPickerFunc(prevVal,...)
	local r, g, b, a;
	if (prevVal) then
		r, g, b, a  = unpack(prevVal);
	else
		r, g, b = CPF:GetColorRGB();
		a = (1 - OpacitySliderFrame:GetValue());
	end
	local frame = CPF.frame;
	if (frame.option == CPF.option) then
		frame.texture:SetVertexColor(r,g,b,a);
	end
	if (frame.option.subType == 2) then
		frame.factory:ChangeSettingFunc(frame.option.var,format("|c%.2x%.2x%.2x%.2x",a * 255,r * 255,g * 255,b * 255));
	else
		frame.factory:ChangeSettingFunc(frame.option.var,{ r, g, b, a });
	end
end

-- OnClick
local function ColorButton_OnClick(self,button)
	local r, g, b, a;
	if (self.option.subType == 2) then
		r, g, b, a = self.factory:HexStringToRGBA(self.factory.cfg[self.option.var]);
	else
		r, g, b, a = unpack(self.factory.cfg[self.option.var]);
	end

	CPF.frame = self;
	CPF.option = self.option;
	CPF.func = ColorButton_ColorPickerFunc;
	CPF.cancelFunc = ColorButton_ColorPickerFunc;
	CPF.opacityFunc = ColorButton_ColorPickerFunc;
	CPF.hasOpacity = true;
	CPF.opacity = (1 - a);
	CPF.previousValues = { r, g, b, a };

	OpacitySliderFrame:SetValue(1 - a);
	CPF:SetColorRGB(r,g,b);
	CPF:Show();
end

-- OnEnter
local function ColorButton_OnEnter(self)
	self.text:SetTextColor(1,1,1);
	self.border:SetVertexColor(1,1,0);
	if (self.option.tip) then
		GameTooltip:SetOwner(self,"ANCHOR_RIGHT");
		GameTooltip:AddLine(self.option.label,1,1,1);
		GameTooltip:AddLine(self.option.tip,nil,nil,nil,1);
		GameTooltip:Show();
	end
end

-- OnLeave
local function ColorButton_OnLeave(self)
	self.text:SetTextColor(1,0.82,0);
	self.border:SetVertexColor(1,1,1);
	GameTooltip:Hide();
end

-- New ColorButton
function AzOptionsFactory:Color()
	local f = CreateFrame("Button",nil,self.owner);
	f:SetWidth(18);
	f:SetHeight(18);
	f:SetScript("OnEnter",ColorButton_OnEnter);
	f:SetScript("OnLeave",ColorButton_OnLeave)
	f:SetScript("OnClick",ColorButton_OnClick);

	f.texture = f:CreateTexture();
	f.texture:SetPoint("TOPLEFT",-1,1);
	f.texture:SetPoint("BOTTOMRIGHT",1,-1);
	f.texture:SetTexture("Interface\\ChatFrame\\ChatFrameColorSwatch");
	f:SetNormalTexture(f.texture);

	f.border = f:CreateTexture(nil,"BACKGROUND");
	f.border:SetPoint("TOPLEFT");
	f.border:SetPoint("BOTTOMRIGHT");
	f.border:SetTexture(1,1,1,1);

	f.text = f:CreateFontString(nil,"ARTWORK","GameFontNormal");
	f.text:SetPoint("LEFT",f,"RIGHT",4,1);

	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                           DropDown Frame                                           --
--------------------------------------------------------------------------------------------------------

-- New DropDown
function AzOptionsFactory:DropDown()
	local f = AzDropDown.CreateDropDown(self.owner,180,true,nil,nil);
	f.text = f:CreateFontString(nil,"ARTWORK","GameFontNormalSmall");
	f.text:SetPoint("LEFT",-302 + f:GetWidth(),0);
	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                              Text Edit                                             --
--------------------------------------------------------------------------------------------------------

-- OnTextChange
local function TextEdit_OnTextChanged(self)
	self.factory:ChangeSettingFunc(self.option.var,self:GetText():gsub("||","|"));
end

-- OnEnterPressed
local function TextEdit_OnEnterPressed(self)
	self:ClearFocus();
end

-- New Text Edit
function AzOptionsFactory:Text()
	local f = CreateFrame("EditBox",GenerateObjectName("EditBox"),self.owner,"InputBoxTemplate");
	f:SetWidth(170);
	f:SetHeight(21);
	f:SetScript("OnTextChanged",TextEdit_OnTextChanged);
	f:SetScript("OnEnterPressed",TextEdit_OnEnterPressed);
	f:SetAutoFocus(nil);

	f.text = f:CreateFontString(nil,"ARTWORK","GameFontNormalSmall");
	f.text:SetPoint("LEFT",-128,1);

	f.factory = self;
	return f;
end